"""Code audit tool for python.

:copyright: 2013 by Kirill Klenov.
:license: MIT, see LICENSE.rst for more details.
"""

__version__ = "0.1.2"
__project__ = "pylava"
__author__ = "Pyfocus community"
__license__ = "MIT"
